//
// Created by lukas on 10.11.20.
//

#pragma once

#include <Ticker.h>
#include <DHT.h>
#include "Pins.h"
#include "SettingsService.h"

class Heating {
public:
    Heating();

    /**
     * initialize the heater
     * @param mode select the operating mode
     * @param settings a settings object is needed for HUMIDITY mode to get percent values set by user
     */
    void init(unsigned mode, const SettingState* settings);

    /**
     * get the last captured humidity value
     * @return humidity value
     */
    float* getLastHum();

    /**
     * get the last captured temperature value
     * @return temperature value
     */
    float* getLastTemp();

    enum MODES {TIME, HUMIDITY};
private:
    Ticker mHeizungTicker;
    Ticker mLuefterTicker;
    Ticker mTurnOffTicker;

    const SettingState* msettings;

    bool mHeatingStatus;

    float lasttemp;
    float lasthum;

    DHT22 sensor;

    /**
     * handle events after successfully received a humidity value
     * - turn on/off of heating
     * - afterrun of fan
     */
    void handleHeatingEvents();

    /**
     * handle events for time-heating mode
     * when to turn on/off heating + fan
     */
    void handleTimeHeater();
};
