//
// Created by lukas on 06.12.20.
//

#include "SettingsService.h"
#include <ESPFS.h>

SettingsService::SettingsService(AsyncWebServer *server, SecurityManager *securityManager) :
        _httpEndpoint(SettingState::read,
                      SettingState::update,
                      this,
                      server,
                      SETTINGS_ENDPOINT_PATH,
                      securityManager,
                      AuthenticationPredicates::IS_AUTHENTICATED) {
    addUpdateHandler([&](const String &originId) { onConfigUpdated(); }, false);
}

void SettingsService::onConfigUpdated() {
    Serial.println("switching led!");

    File settingsFile = ESPFS.open("settings.json", "w+");

    if (settingsFile) {
        Serial.println("writing config file!");
        StaticJsonDocument<256> doc;

        doc["waterOutageWaitDuration"] = _state.waterOutageWaitDuration;
        doc["maxpumpduration"] = _state.maxpumpduration;
        doc["heatUp"] = _state.heatUp;
        doc["heatLow"] = _state.heatLow;
        doc["fanRuntime"] = _state.fanRuntime;

        serializeJson(doc, settingsFile);

        settingsFile.close();
    }
}

void SettingsService::begin() {
    File settingsFile = ESPFS.open("settings.json", "r");

    if (settingsFile) {
        StaticJsonDocument<512> doc;

        // Deserialize the JSON document
        DeserializationError error = deserializeJson(doc, settingsFile);
        if (error) {
            Serial.println(F("Failed to read file, using default configuration"));
            onConfigUpdated();
            return;
        }


        // Copy values from the JsonDocument to the Config
        _state.waterOutageWaitDuration = doc["waterOutageWaitDuration"] | 3600;
        _state.maxpumpduration = doc["maxpumpduration"] | 300;
        _state.heatUp = doc["heatUp"] | 70;
        _state.heatLow = doc["heatLow"] | 65;
        _state.fanRuntime = doc["fanRuntime"] | 60;

        Serial.println("read config file!");

        settingsFile.close();
    }
}

const SettingState* SettingsService::getSettings() {
    return &_state;
}
